==> enterprisemodules-ora_install-5.14.3.tar.gz: enterprisemodules-ora_install-5.14.3/tasks/grid_action.rb <==
#!/opt/puppetlabs/puppet/bin/ruby
#
#
# See the file "LICENSE" for the full license governing this code.
#
#
require 'json'
require 'open3'
require 'puppet'
require 'yaml'
require 'fileutils'
require 'time'

begin
  Puppet.initialize_settings
rescue Puppet::DevError
  # When run in Rspec, this is called multiple times
end

class GridAction

  def self.run
    instance = self.new
    instance.run
  end

  def run()
    @starttime = Time.now.iso8601
    begin
      fetch_input()
      if @action == 'stop'
        log_msg stop
      elsif @action == 'start'
        log_msg start
      elsif @action == 'restart'
        log_msg stop
        log_msg start
      end
      output_message = @message.join("\n")
      output(0, output_message)
    rescue => e
      error_output(1, e.message, "enterprisemodules-ora_install/task-error")
    end
  end

  def fetch_input
    params        = JSON.parse(STDIN.read)
    #
    # Use loadpath specfied in the params
    #
    $LOAD_PATH << "#{params['_installdir']}/easy_type/lib"
    $LOAD_PATH << "#{params['_installdir']}/ora_install/lib"
    require 'easy_type'
    require 'puppet_x/enterprisemodules/ora_install/tasks'
    extend ::PuppetX::EnterpriseModules::OraInstall::Tasks

    #
    # Fetch verbose level
    #
    @verbose = params.fetch('verbose') { 'info' }
    Puppet::Util::Log.level = @verbose

    @action         = params.fetch('action')
    @oracle_homes   = Facter.value('ora_install_homes')
    @current_home   = asm_home
    @sid            = asm_sid
    @asm_instance   = @sid =~ /^\+ASM/
    @cluster        = @sid =~ /^\+ASM[1-9]/
    @format         = params.fetch('format') { 'pretty' }
    error_output(1, "cannot find asm instance on this server.", 'enterprisemodules-ora_install/parameter-error') unless @asm_instance
    @message = []
  end

  def stop
    if @asm_instance
      stop_running_databases
      # stop_asm_listener unless @cluster
      # stop_asm unless @cluster
    else
      stop_listener unless running_on_asm?
      stop_database
    end
  end

  def start
    if @asm_instance
      # start_asm_listener unless @cluster
      # start_asm unless @cluster
      start_stopped_databases
    else
      start_database
      start_listener unless running_on_asm?
    end
  end

  def stop_running_databases
    @oracle_homes.keys.each do |home|
      unless home == @current_home
        if @oracle_homes[home]['defined_sids'].any?
          log_msg "Stop database(s) #{@oracle_homes[home]['defined_sids'].join(",")} from oracle_home #{home}"
          file = home.gsub('/','_')
          statefile = "/tmp/#{file}.txt"
          File.delete(statefile) if File.exist?(statefile)
          if @cluster
            exec("#{@current_home}/bin/srvctl stop home -oraclehome #{home} -node #{hostname} -statefile #{statefile}")
          else
            exec("#{@current_home}/bin/srvctl stop home -oraclehome #{home} -statefile #{statefile}")
          end
        end
      end
    end
    nil
  end

  def stop_asm_listener
    log_msg 'Stop ASM listener'
    exec("#{@current_home}/bin/srvctl stop listener")
  end

  def start_asm_listener
    log_msg 'Start ASM listener'
    exec("#{@current_home}/bin/srvctl start listener")
  end

  def stop_asm
    log_msg 'Stop ASM'
    exec("#{@current_home}/bin/srvctl stop asm -f")
  end

  def start_asm
    log_msg 'Start ASM'
    exec("#{@current_home}/bin/srvctl start asm")
  end

  def start_stopped_databases
    @oracle_homes.keys.each do |home|
      unless home == asm_home
        if @oracle_homes[home]['defined_sids'].any?
          log_msg "Start database(s) #{@oracle_homes[home]['defined_sids'].join(",")} from oracle_home #{home}"
          file = home.gsub('/','_')
          statefile = "/tmp/#{file}.txt"
          unless File.exist?(statefile)
            dbs = @oracle_homes[home]['defined_sids'].map{|x| 'db-' + x.downcase}
            file_content = dbs.join("\n")
            File.open(statefile, 'w') { |file| file.write(file_content) }
          end
          if @cluster
            exec("#{@current_home}/bin/srvctl start home -oraclehome #{home} -node #{hostname} -statefile #{statefile}")
          else
            exec("#{@current_home}/bin/srvctl start home -oraclehome #{home} -statefile #{statefile}")
          end
          File.delete(statefile) if File.exist?(statefile)
        end
      end
    end
    nil
  end

  def hostname
    Facter.value('hostname')
  end

  private

    def exec( command, options =  {} )
      options = {:failonfail => true, :combine => true}.merge(options)
      output = Puppet::Util::Execution.execute(command, options).to_s
      Puppet.debug output
      output
    end

    def ora_settings?
      File.exist?('/etc/ora_setting.yaml')
    end

    def home_for_sid(sid)
      selected_home_info = @oracle_homes.select {|d| @oracle_homes[d].fetch('defined_sids'){[]}.include?(sid)}
      fail "No home found for sid #{sid}" if selected_home_info.empty?
      selected_home = selected_home_info.keys.first
      fail "Sid #{sid} must be running for switch." unless selected_home_info[selected_home].fetch('running_sids') {[]}.include?(sid)
      selected_home
    end

    def asm_home
      @oracle_homes.select {|d| @oracle_homes[d].fetch('defined_sids').grep(/^\+ASM/).any?}.keys.first
    end

    def asm_sid
      @oracle_homes[asm_home]['defined_sids'].first
    end

    # We are a databse sid but running on top of ASM.
    def running_on_asm?
      !asm_home.nil? && @sid !~ /^\+ASM/
    end

    def asm_sid
      @oracle_homes[asm_home]['defined_sids'].first
    end

    def os_user
      uid = File.stat("#{@current_home}/bin/oracle").uid
      Etc.getpwuid(uid).name
    end

    def os_group
      gid = File.stat(@current_home).gid
      Etc.getgrgid(gid).name
    end

    def asm_action(action, home)
      command = "#{home}/bin/srvctl #{action} asm"
      full_command = "export ORACLE_HOME=#{home};export LD_LIBRARY_PATH=#{home}/lib;cd #{home};#{command}"
      exec(full_command, :uid => os_user)
    end

    def unsuccessful?
      @output.scan(@succes_output).empty?
    end

    def sql(command, home)
      if @asm_instance
        command     = "sqlplus /nolog <<-EOF
connect / as sysasm
#{command}
EOF"
      else
        command     = "sqlplus /nolog <<-EOF
connect / as sysdba
#{command}
EOF"
      end

      exec("export ORACLE_HOME=#{home};export PATH=#{home}/bin:$PATH;export ORACLE_SID=#{@sid};export LD_LIBRARY_PATH=#{home}/lib;#{command}", :uid => os_user)
    end

end

GridAction.run if __FILE__ == $0

==> enterprisemodules-ora_install-6.9.6.tar.gz: enterprisemodules-ora_install-6.9.6/tasks/grid_action.rb <==
#!/opt/puppetlabs/puppet/bin/ruby
#
#
# See the file "LICENSE" for the full license governing this code.
#
#
require 'json'
require 'open3'
require 'puppet'
require 'yaml'
require 'fileutils'
require 'time'

begin
  Puppet.initialize_settings
rescue Puppet::DevError
  # When run in Rspec, this is called multiple times
end

class GridAction

  def self.run
    instance = self.new
    instance.run
  end

  def run()
    @starttime = Time.now.iso8601
    begin
      fetch_input()
      if @action == 'stop'
        log_msg stop
      elsif @action == 'start'
        log_msg start
      elsif @action == 'restart'
        log_msg stop
        log_msg start
      end
      output_message = @message.join("\n")
      output(0, output_message)
    rescue => e
      error_output(1, e.message, "enterprisemodules-ora_install/task-error")
    end
  end

  def fetch_input
    params        = JSON.parse(STDIN.read)
    #
    # Use loadpath specfied in the params
    #
    $LOAD_PATH << "#{params['_installdir']}/easy_type/lib"
    $LOAD_PATH << "#{params['_installdir']}/ora_install/lib"
    require 'easy_type'
    require 'puppet_x/enterprisemodules/ora_install/tasks'
    extend ::PuppetX::EnterpriseModules::OraInstall::Tasks

    #
    # Fetch verbose level
    #
    @verbose = params.fetch('verbose') { 'info' }
    Puppet::Util::Log.level = @verbose

    @action         = params.fetch('action')
    @oracle_homes   = Facter.value('ora_install_homes')
    @current_home   = asm_home
    @sid            = asm_sid
    @asm_instance   = @sid =~ /^\+ASM/
    @cluster        = @sid =~ /^\+ASM[1-9]/
    @format         = params.fetch('format') { 'pretty' }
    error_output(1, "cannot find asm instance on this server.", 'enterprisemodules-ora_install/parameter-error') unless @asm_instance
    @message = []
  end

  def stop
    if @asm_instance
      stop_running_databases
      # stop_asm_listener unless @cluster
      # stop_asm unless @cluster
    else
      stop_listener unless running_on_asm?
      stop_database
    end
  end

  def start
    if @asm_instance
      # start_asm_listener unless @cluster
      # start_asm unless @cluster
      start_stopped_databases
    else
      start_database
      start_listener unless running_on_asm?
    end
  end

  def stop_running_databases
    @oracle_homes.keys.each do |home|
      unless home == @current_home
        if @oracle_homes['defined_sids'][home].any?
          log_msg "Stop database(s) #{@oracle_homes['defined_sids'][home].join(",")} from oracle_home #{home}"
          file = home.gsub('/','_')
          statefile = "/tmp/#{file}.txt"
          File.delete(statefile) if File.exist?(statefile)
          if @cluster
            exec("#{@current_home}/bin/srvctl stop home -oraclehome #{home} -node #{hostname} -statefile #{statefile}")
          else
            exec("#{@current_home}/bin/srvctl stop home -oraclehome #{home} -statefile #{statefile}")
          end
        end
      end
    end
    nil
  end

  def stop_asm_listener
    log_msg 'Stop ASM listener'
    exec("#{@current_home}/bin/srvctl stop listener")
  end

  def start_asm_listener
    log_msg 'Start ASM listener'
    exec("#{@current_home}/bin/srvctl start listener")
  end

  def stop_asm
    log_msg 'Stop ASM'
    exec("#{@current_home}/bin/srvctl stop asm -f")
  end

  def start_asm
    log_msg 'Start ASM'
    exec("#{@current_home}/bin/srvctl start asm")
  end

  def start_stopped_databases
    @oracle_homes['defined_sids'].keys.each do |home|
      unless home == asm_home
        if @oracle_homes['defined_sids'][home].any?
          log_msg "Start database(s) #{@oracle_homes['defined_sids'][home].join(",")} from oracle_home #{home}"
          file = home.gsub('/','_')
          statefile = "/tmp/#{file}.txt"
          unless File.exist?(statefile)
            dbs = @oracle_homes['defined_sids'][home].map{|x| 'db-' + x.downcase}
            file_content = dbs.join("\n")
            File.open(statefile, 'w') { |file| file.write(file_content) }
          end
          if @cluster
            exec("#{@current_home}/bin/srvctl start home -oraclehome #{home} -node #{hostname} -statefile #{statefile}")
          else
            exec("#{@current_home}/bin/srvctl start home -oraclehome #{home} -statefile #{statefile}")
          end
          File.delete(statefile) if File.exist?(statefile)
        end
      end
    end
    nil
  end

  def hostname
    Facter.value('hostname')
  end

  private

    def exec( command, options =  {} )
      options = {:failonfail => true, :combine => true}.merge(options)
      output = Puppet::Util::Execution.execute(command, options).to_s
      Puppet.debug output
      output
    end

    def ora_settings?
      File.exist?('/etc/ora_setting.yaml')
    end

    def home_for_sid(sid)
      selected_home_info = @oracle_homes.select {|d| @oracle_homes[d].fetch('defined_sids'){[]}.include?(sid)}
      fail "No home found for sid #{sid}" if selected_home_info.empty?
      selected_home = selected_home_info.keys.first
      fail "Sid #{sid} must be running for switch." unless selected_home_info[selected_home].fetch('running_sids') {[]}.include?(sid)
      selected_home
    end

    def asm_home
      @oracle_homes.select {|d| @oracle_homes[d].fetch('defined_sids').grep(/^\+ASM/).any?}.keys.first
    end

    def asm_sid
      @oracle_homes[asm_home]['defined_sids'].first
    end

    # We are a databse sid but running on top of ASM.
    def running_on_asm?
      !asm_home.nil? && @sid !~ /^\+ASM/
    end

    def asm_sid
      @oracle_homes[asm_home]['defined_sids'].first
    end

    def os_user
      uid = File.stat("#{@current_home}/bin/oracle").uid
      Etc.getpwuid(uid).name
    end

    def os_group
      gid = File.stat(@current_home).gid
      Etc.getgrgid(gid).name
    end

    def asm_action(action, home)
      command = "#{home}/bin/srvctl #{action} asm"
      full_command = "export ORACLE_HOME=#{home};export LD_LIBRARY_PATH=#{home}/lib;cd #{home};#{command}"
      exec(full_command, :uid => os_user)
    end

    def unsuccessful?
      @output.scan(@succes_output).empty?
    end

    def sql(command, home)
      if @asm_instance
        command     = "sqlplus /nolog <<-EOF
connect / as sysasm
#{command}
EOF"
      else
        command     = "sqlplus /nolog <<-EOF
connect / as sysdba
#{command}
EOF"
      end

      exec("export ORACLE_HOME=#{home};export PATH=#{home}/bin:$PATH;export ORACLE_SID=#{@sid};export LD_LIBRARY_PATH=#{home}/lib;#{command}", :uid => os_user)
    end

end

GridAction.run if __FILE__ == $0
